/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw;

import java.io.File;
import java.rmi.RemoteException;

import de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService;
import de.frankmuenster.jameica.orverw.db.server.OrVerwDBServiceImpl;
import de.frankmuenster.jameica.orverw.gui.navigation.OrdnerVerwaltungExtension;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.datasource.db.EmbeddedDatabase;
import de.willuhn.jameica.gui.extension.ExtensionRegistry;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Hauptklasse des Ordnerverwatungs-Plugins.
 *
 * @author Frank Mnster
 */
public class OrdnerVerwaltung extends AbstractPlugin {

    /**
     * Standard Konstruktor.
     */
    public OrdnerVerwaltung() { // NOPMD
	super();
    }

    /**
     * Inits the.
     *
     * @throws ApplicationException
     *             the application exception
     * @see de.willuhn.jameica.plugin.AbstractPlugin#init()
     */
    @Override
    public void init() throws ApplicationException {
	super.init();
	Logger.debug("starting init process for OrdnerVerwaltung"); //$NON-NLS-1$
	// Datenbank Konsistenz berprfen
	this.call(new ServiceCall() {

	    /**
	     * Ein Service call
	     */
	    @Override
	    public void call(final OrVerwDBService service) throws ApplicationException, RemoteException {
		service.checkConsistency();
	    }
	});
	// Menu des Jameica-'Plugins' erweitern
	ExtensionRegistry.register(new OrdnerVerwaltungExtension(), "jameica.start"); //$NON-NLS-1$
	Logger.debug("ending init process for OrdnerVerwaltung"); //$NON-NLS-1$
    }

    /**
     * This method is called only the first time, the plugin is loaded (before
     * executing init()). if your installation procedure was not successfull,
     * throw an ApplicationException. You dont need to implement this function.
     *
     * @throws ApplicationException
     *             the application exception
     * @see de.willuhn.jameica.plugin.AbstractPlugin#install()
     */
    @Override
    public void install() throws ApplicationException {
	// If we are running in client/server mode and this instance
	// is the client, we do not need to create a database.
	// Instead of this we will get our objects via RMI from
	// the server
	if (Application.inClientMode()) {
	    return;
	}
	try {
	    // Let's create an embedded Database
	    final PluginResources res = Application.getPluginLoader().getPlugin(OrdnerVerwaltung.class).getResources();
	    final Manifest manifest = Application.getPluginLoader().getManifest(OrdnerVerwaltung.class);
	    final EmbeddedDatabase embeddedDb = new EmbeddedDatabase(res.getWorkPath() + "/db", "orverw", "orverwview"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	    // create the sql tables.
	    embeddedDb.executeSQLScript(new File(manifest.getPluginDir() + File.separator + "sql", "create.sql")); //$NON-NLS-1$ //$NON-NLS-2$
	    // That's all. Database installed and tables created ;)
	} catch (final Exception e) { // NOPMD
	    Logger.error(I18N.tr("OrdnerVerwaltung.pluginInstallationFailed") + e.getMessage(), e.getCause()); //$NON-NLS-1$
	    throw new ApplicationException(I18N.tr("OrdnerVerwaltung.pluginInstallationFailed"), e); //$NON-NLS-1$
	}
    }

    /**
     * Hilfsmethode zum bequemen Ausfuehren von Methoden auf dem Service.
     *
     * @param call
     *            der Call.
     * @throws ApplicationException
     *             the application exception
     */
    private void call(final ServiceCall call) throws ApplicationException {
	if (Application.inClientMode()) {
	    return; // als Client muessen wir die DB nicht installieren
	}
	OrVerwDBService service = null; // NOPMD
	try {
	    // Da die Service-Factory zu diesem Zeitpunkt noch nicht da ist,
	    // erzeugen
	    // wir uns eine lokale Instanz des Services.
	    service = new OrVerwDBServiceImpl();
	    service.start();
	    call.call(service);
	} catch (final ApplicationException ae) { // NOPMD
	    throw ae;
	} catch (final RemoteException e) {
	    Logger.error(I18N.tr("OrdnerVerwaltung.initDbFailed"), e); //$NON-NLS-1$
	    final StringBuilder msg = new StringBuilder(536);
	    msg.append(I18N.tr("OrdnerVerwaltung.initDbFailed1")).append("\n\n").append(e.getMessage()) //$NON-NLS-1$ //$NON-NLS-2$
		    .append("\n\n") //$NON-NLS-1$
		    .append(I18N.tr("OrdnerVerwaltung.initDbFailed2") //$NON-NLS-1$
			    + I18N.tr("OrdnerVerwaltung.initDbFailed3") //$NON-NLS-1$
			    + I18N.tr("OrdnerVerwaltung.initDbFailed4") //$NON-NLS-1$
			    + I18N.tr("OrdnerVerwaltung.initDbFailed5") //$NON-NLS-1$
			    + I18N.tr("OrdnerVerwaltung.initDbFailed6")); //$NON-NLS-1$
	    throw new ApplicationException(msg.toString(), e);
	} finally {
	    if (service != null) {
		try {
		    service.stop(true);
		} catch (final RemoteException e) {
		    Logger.error(I18N.tr("OrdnerVerwaltung.initDbFailed7"), e); //$NON-NLS-1$
		}
	    }
	}
    }

    /**
     * Hilfsmethode zum bequemen Ausfuehren von Aufrufen auf dem Service.
     */
    private interface ServiceCall {

	/**
	 * Call.
	 *
	 * @param service
	 *            the service
	 * @throws ApplicationException
	 *             the application exception
	 * @throws RemoteException
	 *             the remote exception
	 */
	void call(OrVerwDBService service) throws ApplicationException, RemoteException;
    }
}
