/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.orverw.parts.labelpart;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

import de.willuhn.jameica.gui.Part;

/**
 * Ein Texteingabefeld bei dem die Gr&ouml;&szlig;e, der Schrifttyp und Farben angegeben werden k&ouml;nnen. Dazu gibt
 * es noch einen Listener, der benachrichtigt, wenn der Text ver&auml;ndert wurde.
 *
 * @author Frank Mnster
 * @since 1.0
 */
public class LabelInputText implements Part {

    /**
     * Das Standartabstand
     */
    private static final int INDENT = 15;

    /**
     * Das eigentliche Eingabe Widget
     */
    private transient StyledText inputText;

    /**
     * Der Inhalt des Eingabefeldes. Wird bentigt um nderungen zu sehen.
     */
    private String text;

    /**
     * Die Gre des Eingabefeldes
     */
    private Point size;

    /**
     * Der Schrifttyp des Textes
     */
    private Font font;

    /**
     * Die Textfarbe
     */
    private Color foreground;

    /**
     * Die Hintergrundfarbe
     */
    private Color background;

    /**
     * Die Textausrichtung
     */
    private int textAlignment;

    /**
     * Die Listener fr einen Button-Click-Event
     */
    private final transient Set<LabelInputTextChangeListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Creates a new LabelInputText with the size.
     *
     * @param size
     *            the initial size
     */
    public LabelInputText(final Point size) {

        this(size, SWT.LEFT);
    }

    /**
     * Creates a new LabelInputText with the size and text alignment.
     *
     * @param size
     *            the initial size
     * @param textAlignment
     *            the initial text alignment
     */
    public LabelInputText(final Point size, final int textAlignment) {

        this.size = size;
        this.textAlignment = textAlignment;

    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            der hinzugefgt werden soll
     */
    public final void addListener(final LabelInputTextChangeListener listener) {

        this.listeners.add(listener);
    }

    /**
     * @return background
     */
    public Color getBackground() {

        return this.inputText.getBackground();
    }

    /**
     * @return font
     */
    public Font getFont() {

        return this.inputText.getFont();
    }

    /**
     * @return fontColor
     */
    public Color getForeground() {

        return this.inputText.getForeground();
    }

    /**
     * @return size
     */
    public Point getSize() {

        return this.size;
    }

    /**
     * @return text
     */
    public String getText() {

        return this.text;
    }

    /**
     * @return textAlignment
     */
    public int getTextAlignment() {

        return this.textAlignment;
    }

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe des angeklickten Buttons
     */
    public final void notifyListeners() {

        for (final LabelInputTextChangeListener listener : this.listeners) {
            listener.notifytextChanged();
        }
    }

    /**
     * @param parent
     */
    @Override
    public void paint(final Composite parent) {

        if (this.inputText != null) {
            return;
        }

        this.inputText = new StyledText(parent, SWT.NONE);
        this.inputText.addKeyListener(new MyKeyListener());
        this.inputText.addFocusListener(new MyFocusListener());

        // GridLayout
        final GridData gridData = new GridData();
        gridData.heightHint = this.size.y;
        gridData.widthHint = this.size.x - 2 * LabelInputText.INDENT;
        this.inputText.setLayoutData(gridData);

        this.inputText.setLeftMargin(LabelInputText.INDENT);
        this.inputText.setRightMargin(LabelInputText.INDENT);
        this.inputText.setWordWrap(true);
        this.inputText.setAlignment(this.getTextAlignment());

        if (this.text != null) {
            this.inputText.setText(this.text);
        }
        if (this.font != null) {
            this.inputText.setFont(this.font);
        }
        if (this.foreground != null) {
            this.inputText.setForeground(this.foreground);
        }
        if (this.background != null) {
            this.inputText.setBackground(this.background);
        }

    }

    /**
     * Entfernd einen Listener aus der Liste
     *
     * @param listener
     *            der entfernt werden soll
     */
    public final void removeListener(final LabelInputTextChangeListener listener) {

        this.listeners.remove(listener);
    }

    /**
     * @param background
     *            das zu setzende Objekt background
     */
    public void setBackground(final Color background) {

        if (background != null) {
            this.background = background;
            if (this.inputText != null) {
                this.inputText.setBackground(background);
            }
        }
    }

    /**
     * @param font
     *            das zu setzende Objekt font
     */
    public void setFont(final Font font) {

        if (font != null) {
            this.font = font;
            if (this.inputText != null) {
                this.inputText.setFont(font);
            }
        }
    }

    /**
     * @param color
     *            das zu setzende Objekt fontColor
     */
    public void setForeground(final Color color) {

        if (color != null) {
            this.foreground = color;
            if (this.inputText != null) {
                this.inputText.setForeground(color);
            }
        }
    }

    /**
     * @param size
     *            das zu setzende Objekt size
     */
    public void setSize(final Point size) {

        if (size != null) {
            this.size = size;
        }
    }

    /**
     * @param text
     *            das zu setzende Objekt text
     */
    public void setText(final String text) {

        if (text != null) {
            this.text = text;
            if (this.inputText != null) {
                this.inputText.setText(text);
            }
        }
    }

    /**
     * @param textAlignment
     *            die zu setzende Textausrichtung
     */
    public void setTextAlignment(final int textAlignment) {

        this.textAlignment = textAlignment;
        if (this.inputText != null) {
            this.inputText.setAlignment(textAlignment);
        }
    }

    /**
     * Privater FocusListener
     *
     * @author Frank Mnster
     */
    private class MyFocusListener implements FocusListener {

        /**
         * Returns a focus listener to store the text of the label in case it gains the focus to see changes
         */

        public MyFocusListener() {

            // intentionally left blank
        }

        /**
         * @see org.eclipse.swt.events.FocusListener#focusGained(org.eclipse.swt.events.FocusEvent)
         */
        @Override
        public void focusGained(final FocusEvent arg0) {

            // nothing to do
        }

        /**
         * @see org.eclipse.swt.events.FocusListener#focusLost(org.eclipse.swt.events.FocusEvent)
         */
        @Override
        public void focusLost(final FocusEvent arg0) {

            // Update text info
            LabelInputText.this.text = LabelInputText.this.inputText.getText();
        }
    }

    /**
     * Privater KeyListener
     *
     * @author Frank Mnster
     */
    private class MyKeyListener implements KeyListener {

        /**
         * Returns a key listener to be added to the label to notify registers label listeners
         */
        public MyKeyListener() {

            // intentionally left blank
        }

        /**
         * @see org.eclipse.swt.events.KeyListener#keyPressed(org.eclipse.swt.events.KeyEvent)
         */
        @Override
        public void keyPressed(final KeyEvent arg0) {

            if (!LabelInputText.this.text.equals(LabelInputText.this.inputText.getText())) {
                LabelInputText.this.notifyListeners();
            }
        }

        /**
         * @see org.eclipse.swt.events.KeyListener#keyReleased(org.eclipse.swt.events.KeyEvent)
         */
        @Override
        public void keyReleased(final KeyEvent arg0) {

            // nothing to do
        }
    }

}
