/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.orverw.parts.labelpart;

import java.rmi.RemoteException;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelButtonListener;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelGeneralButtons;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelInputButtons;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelPartButtonsEnum;
import de.frankmuenster.jameica.orverw.parts.labelpart.trenner.BarTrenner;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.SimpleContainer;

/**
 * Das Part f&uuml;r den Ordnerr&uuml;cken.
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.GodClass", "PMD.TooManyFields" })
public final class LabelPart implements Part, LabelButtonListener, LabelInputTextChangeListener {

    /**
     * Die H&ouml;he der oberen Linie
     */
    public static final int TB_HEIGHT = 40;

    /**
     * Das obere Margin der oberen Linie
     */
    public static final int TB_TOP_MARGIN = 30;

    /**
     * Die H&ouml;he der mittleren Linie
     */
    public static final int MB_HEIGHT = 20;

    /**
     * Das obere Margin der mittleren Linie
     */
    public static final int MB_TOP_MARGIN = 10;

    /**
     * Die Linienh&ouml;e
     */
    public static final int BAR_HEIGHT = 6;

    /**
     * Die default Titeltextausrichtung
     */
    private static final int TITEL_ALIGNMENT = SWT.CENTER;

    /**
     * Der Hauptcontainer f&uuml;r den Ordnerr&uuml;cken
     */
    private transient SimpleContainer label;

    /**
     * background colour
     */
    private Color bgColour;

    /**
     * Die obere Linie
     */
    private final transient BarTrenner bar1;

    /**
     * Die untere Linie
     */
    private final transient BarTrenner bar2;

    /**
     * Sichtbarkeit der Linien
     */
    private boolean barVisible;

    /**
     * Farbe der Linien
     */
    private Color barColour;

    /**
     * Eingabefeld f&uuml;r den Titeltext
     */
    private final transient LabelInputText titelTextPart;

    /**
     * Der Text des Titelfeldes
     */
    private String titelText;

    /**
     * Eingabefeld f&uuml;r den Hauptbereiches
     */
    private final transient LabelInputText contentTextPart;

    /**
     * Der Text des Hauptbereiches
     */
    private String contentText;

    /**
     * Eingabefeld f&uuml;r den Fu&szlig;nereich
     */
    private final transient LabelInputText footerTextPart;

    /**
     * Des des Fu&szlig;nereiches
     */
    private String footerText;

    /**
     * Label f&uuml;r die ID
     */
    private transient CLabel idCLabel;

    /**
     * Der Text der ID
     */
    private String idText;

    /**
     * Sichtbarkeit der ID.
     */
    private boolean idVisible;

    /**
     * Die Breite des Labels
     */
    private int width;

    /**
     * Die Berechnete Gr&ouml;&szlig;e des Titeltextfeldes
     */
    private transient Point titleSize;

    /**
     * Die Berechnete Gr&ouml;&szlig;e des Haupttextfeldes
     */
    private transient Point contentSize;

    /**
     * Die Berechnete Gr&ouml;&szlig;e des Fu&szlig;textfeldes
     */
    private transient Point footerSize;

    /**
     * Die Listener fr einen Button-Click-Event
     */
    private final transient Set<LabelPartListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Erstellt eine neue Ordnerbeschriftung mit der angegeben Breite.
     *
     * @param size
     *            - Breite des Labels in Millimeter
     * @param listener
     *            - Listener fr Knpfe
     */
    public LabelPart(final LabelSizeEnum size, final LabelPartListener listener) {

        if (size == null) {
            this.width = LabelSizeEnum.getDefault().getWidth();
        } else {
            this.width = size.getWidth();
        }
        this.barVisible = true;
        this.idVisible = true;

        this.addListener(listener);

        this.calculateSizes();
        final Point topBarSize = new Point(this.titleSize.x, LabelPart.TB_HEIGHT + LabelPart.BAR_HEIGHT);
        this.bar1 = new BarTrenner(topBarSize, LabelPart.TB_TOP_MARGIN, LabelPart.BAR_HEIGHT, this.getBarColour(),
                this.getBgColour());
        this.titelTextPart = new LabelInputText(this.titleSize, LabelPart.TITEL_ALIGNMENT);
        this.titelTextPart.addListener(this);
        final Point middleBarSize = new Point(this.titleSize.x, LabelPart.MB_HEIGHT + LabelPart.BAR_HEIGHT);
        this.bar2 = new BarTrenner(middleBarSize, LabelPart.MB_TOP_MARGIN, LabelPart.BAR_HEIGHT, this.getBarColour(),
                this.getBgColour());
        this.contentTextPart = new LabelInputText(this.contentSize);
        this.contentTextPart.addListener(this);
        this.footerTextPart = new LabelInputText(this.footerSize);
        this.footerTextPart.addListener(this);

    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            der hinzugefgt werden soll
     */
    public void addListener(final LabelPartListener listener) {

        this.listeners.add(listener);
    }

    /**
     * Liefert die Textausrichtung des angegebenen Eingabefeldes.
     *
     * @param section
     *            das Eingabefeld
     * @return Die Textausrichtung
     */
    public int getAlignment(final LabelPartSectionsEnum section) {

        int alignment;

        if (section == null) {
            alignment = SWT.LEFT;
        } else {
            switch (section) {
                case CONTENT:
                    alignment = this.contentTextPart.getTextAlignment();
                    break;
                case FOOTER:
                    alignment = this.footerTextPart.getTextAlignment();
                    break;
                case TITLE:
                    alignment = this.titelTextPart.getTextAlignment();
                    break;
                default:
                    alignment = SWT.LEFT;
                    break;
            }
        }
        return alignment;
    }

    /**
     * Liefert die Farbe der Linien. Wurde noch keine Farbe gesetzt wird sie auf RGB(0, 0, 240) gesetzt.
     *
     * @return Die Farbe der Linien
     */
    public Color getBarColour() {

        if (this.barColour != null) {
            this.barColour = new Color(GUI.getDisplay(), 0, 0, 240);
        }
        return this.barColour;
    }

    /**
     * Liefert die Sichtbarkeit der Linien
     *
     * @return Die Sichtbarkeit der Linien
     */
    public Boolean getBarVisible() {

        return this.barVisible;
    }

    /**
     * Liefert die Hintergrundfarbe des Labels. Wurde noch keine gesetzt wird sie auf RGB(255, 255, 224) gesetzt.
     *
     * @return Die Hintergrundfarbe des Labels
     */
    public Color getBgColour() {

        if (this.bgColour == null) {
            this.bgColour = new Color(GUI.getDisplay(), 255, 255, 224);
        }
        return this.bgColour;
    }

    /**
     * Liefert den Text des Hauptfeldes
     *
     * @return Den Text des Hauptfeldes
     */
    public String getContentText() {

        return this.contentTextPart.getText();
    }

    /**
     * Liefert die Textfarbe in einem angegeben Eingabefeld.
     *
     * @param section
     *            das Eingabefeld
     * @return Die Textfarbe
     */
    public Color getFgColour(final LabelPartSectionsEnum section) {

        Color colour;

        if (section == null) {
            colour = GUI.getShell().getForeground();
        } else {
            switch (section) {
                case CONTENT:
                    colour = this.contentTextPart.getForeground();
                    break;
                case FOOTER:
                    colour = this.footerTextPart.getForeground();
                    break;
                case TITLE:
                    colour = this.titelTextPart.getForeground();
                    break;
                default:
                    colour = GUI.getShell().getForeground();
                    break;
            }
        }

        return colour;
    }

    /**
     * Liefert den Schrifttyp ein dem angegeben Eingabefeld.
     *
     * @param section
     *            das Eingabefeld
     * @return Den Schrifttyp
     */
    public Font getFont(final LabelPartSectionsEnum section) {

        Font font;

        if (section == null) {
            font = GUI.getShell().getFont();
        } else {
            switch (section) {
                case CONTENT:
                    font = this.contentTextPart.getFont();
                    break;
                case FOOTER:
                    font = this.footerTextPart.getFont();
                    break;
                case TITLE:
                    font = this.titelTextPart.getFont();
                    break;
                default:
                    font = GUI.getShell().getFont();
                    break;
            }
        }
        return font;
    }

    /**
     * Liefert den Text des Fu&szlig;feldes
     *
     * @return Den Text des Fu&szlig;feldes
     */
    public String getFooterText() {

        return this.footerTextPart.getText();
    }

    /**
     * Liefert die ID
     *
     * @return Die ID
     */
    public String getIdText() {

        return this.idText;
    }

    /**
     * Liefert den Text des Titelfeldes
     *
     * @return Den Text des Titelfeldes
     */
    public String getTitelText() {

        return this.titelTextPart.getText();
    }

    /**
     * Liefert die Breite des Labels
     *
     * @return die Breite des Labels
     */
    public int getWidth() {

        return this.width;
    }

    /**
     * Liefert die Sichtbarkeit der ID
     *
     * @return Die Sichtbarkeit der ID
     */
    public boolean isIdVisible() {

        return this.idVisible;
    }

    /**
     * @see de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelButtonListener#notifyInputButtonClicked(de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartSectionsEnum,
     *      de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelPartButtonsEnum)
     */
    @Override
    public void notifyInputButtonClicked(final LabelPartSectionsEnum section, final LabelPartButtonsEnum button) {

        if (section != null) {
            Font font;
            Color colour;

            switch (section) {
                case CONTENT:
                    font = this.contentTextPart.getFont();
                    colour = this.contentTextPart.getForeground();
                    break;
                case FOOTER:
                    font = this.footerTextPart.getFont();
                    colour = this.footerTextPart.getForeground();
                    break;
                case TITLE:
                    font = this.titelTextPart.getFont();
                    colour = this.titelTextPart.getForeground();
                    break;
                case GENERAL:
                    font = null; // NOPMD
                    switch (button) {
                        case BAR_COLOUR:
                            colour = this.getBarColour();
                            break;
                        case BACKGROUND_COLOUR:
                            colour = this.getBgColour();
                            break;
                        default:
                            colour = null; // NOPMD
                            break;
                    }
                    break;
                default:
                    font = new Font(GUI.getDisplay(), new FontData());
                    colour = new Color(GUI.getDisplay(), new RGB(0, 0, 0));
                    break;
            }
            this.notifyListenersButtonClicked(new LabelPartButtonObject(section, button, font, colour));

        }
    }

    /**
     * @see de.frankmuenster.jameica.orverw.parts.labelpart.LabelInputTextChangeListener#notifytextChanged()
     */
    @Override
    public void notifytextChanged() {

        this.notifyListenersTextChanged();
    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        this.label = new SimpleContainer(parent, false);
        this.paintPart();

    }

    /**
     * Entfernd einen Listener aus der Liste
     *
     * @param listener
     *            der entfernt werden soll
     */
    public void removeListener(final LabelPartListener listener) {

        this.listeners.remove(listener);
    }

    /**
     * Setzt die Textausrichtung in einem angegeben Eingabefeld.
     *
     * @param section
     *            das Eingabefeld.
     * @param alignment
     *            die Textausrichtung
     */
    public void setAlignment(final LabelPartSectionsEnum section, final int alignment) {

        if (section != null) {
            switch (section) {
                case CONTENT:
                    this.contentTextPart.setTextAlignment(alignment);
                    break;
                case FOOTER:
                    this.footerTextPart.setTextAlignment(alignment);
                    break;
                case TITLE:
                    this.titelTextPart.setTextAlignment(alignment);
                    break;
                default:
                    break;
            }
        }
    }

    /**
     * Setzt die Farbe der Linien
     *
     * @param colour
     *            die Farbe der Linien
     */
    public void setBarColour(final Color colour) {

        if (colour != null) {
            this.barColour = colour;
            if (this.bar1 != null) {
                this.bar1.setBarColor(colour);
            }
            if (this.bar2 != null) {
                this.bar2.setBarColor(colour);
            }
        }
    }

    /**
     * Setzt die Sichtbarkeit der Linien
     *
     * @param lineVisible
     *            die Sichtbarkeit der Linien
     */
    public void setBarVisible(final boolean lineVisible) {

        this.barVisible = lineVisible;
        if (this.bar1 != null) {
            this.bar1.setVisible(this.barVisible);
        }
        if (this.bar2 != null) {
            this.bar2.setVisible(this.barVisible);
        }
    }

    /**
     * Setzt die Hintergrundfarbe des Labels.
     *
     * @param colour
     *            die Hintergrundfarbe des Labels.
     */
    public void setBgColour(final Color colour) {

        if (colour != null) {
            this.bgColour = colour;
            if (this.label != null) {
                this.setBackgrounds();
            }
        }
    }

    /**
     * Setzt den Text des Hauptfeldes
     *
     * @param content
     *            der Text des Hauptfeldes
     */
    public void setContentText(final String content) {

        if (content != null) {
            this.contentText = content;
            if (this.contentTextPart != null) {
                this.contentTextPart.setText(content);
            }
        }
    }

    /**
     * Setzt den Schrifttyp in einem angegeben Eingabefeld.
     *
     * @param section
     *            das Eingabefeld
     * @param font
     *            den Schrifttyp
     */
    public void setFont(final LabelPartSectionsEnum section, final Font font) {

        if (section != null) {
            switch (section) {
                case CONTENT:
                    this.contentTextPart.setFont(font);
                    break;
                case FOOTER:
                    this.footerTextPart.setFont(font);
                    break;
                case TITLE:
                    this.titelTextPart.setFont(font);
                    break;
                default:
                    break;
            }
        }
    }

    /**
     * Setzt den Text des Fu&szlig;feldes
     *
     * @param footer
     *            Der Text des Fu&szlig;feldes
     */
    public void setFooterText(final String footer) {

        if (footer != null) {
            this.footerText = footer;
            if (this.footerTextPart != null) {
                this.footerTextPart.setText(footer);
            }
        }
    }

    /**
     * Setzt die Textfarbe in einem angegeben Eingabefeld.
     *
     * @param section
     *            das Eingabefeld
     * @param color
     *            die Farbe
     */
    public void setForegroundColor(final LabelPartSectionsEnum section, final Color color) {

        if (section != null) {
            switch (section) {
                case CONTENT:
                    this.contentTextPart.setForeground(color);
                    break;
                case FOOTER:
                    this.footerTextPart.setForeground(color);
                    break;
                case TITLE:
                    this.titelTextPart.setForeground(color);
                    break;
                default:
                    break;
            }
        }

    }

    /**
     * Setzt die ID
     *
     * @param idText
     *            Die ID
     */
    public void setIdText(final String idText) {

        this.idText = idText;
        if (this.idCLabel != null) {
            this.fillText();
        }
    }

    /**
     * Setzt die Sichtbarkeit der ID
     *
     * @param visible
     *            die Sichtbarkeit der ID
     */
    public void setIdVisible(final boolean visible) {

        this.idVisible = visible;
        if (this.idCLabel != null) {
            this.idCLabel.setVisible(visible);
        }
    }

    /**
     * Setzt den Text des Titelfeldes
     *
     * @param titel
     *            der Text des Titelfeldes
     */
    public void setTitelText(final String titel) {

        if (titel != null) {
            this.titelText = titel;
            if (this.titelTextPart != null) {
                this.titelTextPart.setText(titel);
            }
        }
    }

    /**
     * @param titelAlignment
     */
    public void setTitleAlignment(final int titelAlignment) {

        this.titelTextPart.setTextAlignment(titelAlignment);
    }

    /**
     * Setzt die Breite des Labels
     *
     * @param width
     *            Die Breite des Labels
     */
    public void setWidth(final int width) {

        this.width = width;
    }

    /**
     * Berechnet die Gr&ouml;&szlig; der Eingabefelder
     */
    private void calculateSizes() {

        int locationX;
        int locationY;
        final Point dpi = GUI.getDisplay().getDPI();

        final int marginLeftRight = (int) (5 * 0.039 * dpi.x);
        locationX = (int) (this.width * 0.039 * dpi.x);
        locationY = (int) (190 * 0.039 * dpi.y);
        new Point(locationX, locationY);

        locationX -= 2 * marginLeftRight;
        locationY = (int) (20 * 0.039 * dpi.y);
        this.titleSize = new Point(locationX, locationY);

        locationY = (int) (80 * 0.039 * dpi.y);
        this.contentSize = new Point(locationX, locationY);

        locationY = (int) (20 * 0.039 * dpi.y);
        this.footerSize = new Point(locationX, locationY);

    }

    /**
     * Aktualisiert die Texte in den Eingabefeldern
     */
    private void fillText() {

        if (this.titelTextPart != null) {
            this.titelTextPart.setText(this.titelText);
        }
        if (this.contentTextPart != null) {
            this.contentTextPart.setText(this.contentText);
        }
        if (this.footerTextPart != null) {
            this.footerTextPart.setText(this.footerText);
        }
        if (this.idCLabel != null) {
            this.idCLabel.setText(this.idText);
        }

    }

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe des angeklickten Buttons
     *
     * @param button
     */
    private void notifyListenersButtonClicked(final LabelPartButtonObject button) {

        for (final LabelPartListener listener : this.listeners) {
            listener.notifyButtonClicked(button);
        }
    }

    /**
     * Benachrichtigen der registrierten Listeners das der Text ge&auml;ndert wurde.
     */
    private void notifyListenersTextChanged() {

        for (final LabelPartListener listener : this.listeners) {
            listener.notifyLabelTextChanged();
        }
    }

    /**
     * Die eigentliche Zeichenroutine.
     */
    private void paintPart() {

        final GridLayout gridLayout = new GridLayout();
        gridLayout.marginWidth = 0;
        gridLayout.marginHeight = 0;
        gridLayout.verticalSpacing = 0;
        gridLayout.numColumns = 5;
        this.label.getComposite().setLayout(gridLayout);

        // leere Spalte links
        final Label emptyLeft = new Label(this.label.getComposite(), SWT.NONE);
        emptyLeft.setText(StringUtils.EMPTY);
        GridData gridData = new GridData();
        gridData.horizontalIndent = 0;
        gridData.verticalIndent = 0;
        gridData.verticalSpan = 6;
        gridData.verticalAlignment = GridData.FILL;
        gridData.widthHint = 30;
        emptyLeft.setLayoutData(gridData);

        this.bar1.paint(this.label.getComposite());
        new Label(this.label.getComposite(), SWT.NONE);
        // leere Spalte rechts (vorletzte)
        final Label emptyRight = new Label(this.label.getComposite(), SWT.NONE);
        emptyRight.setText(StringUtils.EMPTY);
        gridData = new GridData();
        gridData.horizontalIndent = 0;
        gridData.verticalIndent = 0;
        gridData.verticalSpan = 6;
        gridData.verticalAlignment = GridData.FILL;
        gridData.widthHint = 30;
        emptyRight.setLayoutData(gridData);

        new Label(this.label.getComposite(), SWT.NONE);

        this.titelTextPart.paint(this.label.getComposite());
        this.titelTextPart.setText(I18N.tr("LabelPart.titleSampleText")); //$NON-NLS-1$
        new LabelInputButtons(this.label.getComposite(), LabelPartSectionsEnum.TITLE, this);

        final LabelGeneralButtons generalButtons = new LabelGeneralButtons(this.label.getComposite(),
                LabelPartSectionsEnum.GENERAL, this);
        gridData = new GridData();
        gridData.horizontalIndent = 0;
        gridData.verticalIndent = 0;
        gridData.verticalSpan = 4;
        gridData.verticalAlignment = GridData.FILL;
        generalButtons.setLayoutData(gridData);

        this.bar2.paint(this.label.getComposite());
        new Label(this.label.getComposite(), SWT.NONE);

        this.contentTextPart.paint(this.label.getComposite());
        this.contentTextPart.setText(I18N.tr("LabelPart.contentSampleText")); //$NON-NLS-1$
        new LabelInputButtons(this.label.getComposite(), LabelPartSectionsEnum.CONTENT, this);

        this.footerTextPart.paint(this.label.getComposite());
        new LabelInputButtons(this.label.getComposite(), LabelPartSectionsEnum.FOOTER, this);

        this.idCLabel = new CLabel(this.label.getComposite(), SWT.BORDER);
        gridData = new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1);
        gridData.widthHint = 30;
        this.idCLabel.setLayoutData(gridData);
        this.idCLabel.setRightMargin(5);
        this.idCLabel.setAlignment(SWT.RIGHT);
        this.idCLabel.setVisible(this.isIdVisible());

        // Texte
        this.fillText();
        this.setBackgrounds();

        this.label.getComposite().pack();
        this.label.getComposite().layout();

    }

    /**
     * Setzt die Hintergrundfarbe aller Widgets
     */
    private void setBackgrounds() {

        this.bar1.setBackground(this.getBgColour());
        this.bar2.setBackground(this.getBgColour());
        this.titelTextPart.setBackground(this.getBgColour());
        this.contentTextPart.setBackground(this.getBgColour());
        this.footerTextPart.setBackground(this.getBgColour());
        this.idCLabel.setBackground(this.getBgColour());
    }

}
