/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.parts.labelpart.buttons;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartSectionsEnum;

/**
 * Abstract class for a area to hold the text/bar style buttons for a section
 *
 * @author Frank Mnster
 * @since 1.0
 */
public abstract class AbstractButtonArea extends Composite implements SelectionListener {

    /**
     * Der Standartabstand zwischen den Button und zum Rand
     */
    protected static final int MARGIN = 5;

    /**
     * Der Bereich zu dem die Buttons geh&ouml;ren.
     */
    protected final transient LabelPartSectionsEnum section;

    /**
     * Die Listener fr einen Button-Click-Event
     */
    private final transient Set<LabelButtonListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Creates a ButtonArea in the given composite for the given section
     *
     * @param parent
     *            das SWT-Eltern_Composite in dem die Buttons gemalt werden
     * @param section
     *            der Bereich zu dem die Buttons geh&ouml;ren. (LabelPartSectionsEnum)
     * @param listener
     *            der Listener, der benachrichtigt wird, wenn ein Button gedr&uuml;ckt wurde.
     */
    public AbstractButtonArea(final Composite parent, final LabelPartSectionsEnum section,
            final LabelButtonListener listener) {

        super(parent, SWT.NONE);
        this.addListener(listener);
        this.section = section;

        this.drawButtons(this);

        // Layout anpassen
        final GridData gridData = new GridData(SWT.LEFT, SWT.TOP, true, true, 1, 1);
        gridData.heightHint = this.getSize().y;
        gridData.widthHint = this.getSize().x;
        this.setLayoutData(gridData);

    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            der hinzugefgt werden soll
     */
    public final void addListener(final LabelButtonListener listener) {

        this.listeners.add(listener);
    }

    /**
     * Liefert den gesetzt Bereich. M&uuml;sste eigentlich immer LabelPartSectionsEnum.GENERAL sein.
     *
     * @return den gesetzten LabelPartSectionsEnum Bereich
     */
    public LabelPartSectionsEnum getSection() {

        return this.section;
    }

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe des angeklickten Buttons
     *
     * @param button
     *            the button type that was clicked
     */
    public final void notifyListeners(final LabelPartButtonsEnum button) {

        for (final LabelButtonListener listener : this.listeners) {
            listener.notifyInputButtonClicked(this.section, button);
        }
    }

    /**
     * Entfernt einen Listener aus der Liste
     *
     * @param listener
     *            der entfernt werden soll
     */
    public final void removeListener(final LabelButtonListener listener) {

        this.listeners.remove(listener);
    }

    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
     */
    @Override
    public final void widgetDefaultSelected(final SelectionEvent evt) { // NOPMD

        // Nothing to do here
    }

    /**
     * Eventlistner wenn ein Widget ausgew&auml;hlt wurde.
     *
     * @param evt
     *            das SelectionEvent
     */
    @Override
    public void widgetSelected(final SelectionEvent evt) {

        final Object source = evt.getSource();
        if (source instanceof Button) {
            final LabelPartButtonsEnum buttonSource = (LabelPartButtonsEnum) ((Button) source).getData();
            this.notifyListeners(buttonSource);
        }
    }

    /**
     * draws the button area and assign the corresponding {@link LabelPartButtonsEnum} to each of them
     *
     * @param composite
     *            in which the button need to be drawn
     */
    protected abstract void drawButtons(Composite composite);

}
